@@ -1,7 +1,6 @@
 use std::io::Write;
 use std::path::{Path, PathBuf};
 
-use clap::arg_enum;
 use serde::{Deserialize, Serialize};
 
 use crate::error::FatalError;
@@ -16,17 +15,12 @@ pub struct Config {
     pub push_remote: Option<String>,
     pub registry: Option<String>,
     pub release: Option<bool>,
-    pub disable_release: Option<bool>,
     pub publish: Option<bool>,
-    pub disable_publish: Option<bool>,
     pub verify: Option<bool>,
-    pub no_verify: Option<bool>,
     pub push: Option<bool>,
-    pub disable_push: Option<bool>,
     pub push_options: Option<Vec<String>>,
     pub dev_version_ext: Option<String>,
     pub dev_version: Option<bool>,
-    pub no_dev_version: Option<bool>,
     pub shared_version: Option<bool>,
     pub consolidate_commits: Option<bool>,
     pub consolidate_pushes: Option<bool>,
@@ -39,13 +33,59 @@ pub struct Config {
     pub tag_prefix: Option<String>,
     pub tag_name: Option<String>,
     pub tag: Option<bool>,
-    pub disable_tag: Option<bool>,
     pub enable_features: Option<Vec<String>>,
     pub enable_all_features: Option<bool>,
     pub dependent_version: Option<DependentVersion>,
 }
 
 impl Config {
+    pub fn new() -> Self {
+        Default::default()
+    }
+
+    pub fn from_defaults() -> Self {
+        let empty = Config::new();
+        Config {
+            allow_branch: Some(
+                empty
+                    .allow_branch()
+                    .map(|s| s.to_owned())
+                    .collect::<Vec<String>>(),
+            ),
+            sign_commit: Some(empty.sign_commit()),
+            sign_tag: Some(empty.sign_tag()),
+            push_remote: Some(empty.push_remote().to_owned()),
+            registry: empty.registry().map(|s| s.to_owned()),
+            release: Some(empty.release()),
+            publish: Some(empty.publish()),
+            verify: Some(empty.verify()),
+            push: Some(empty.push()),
+            push_options: Some(
+                empty
+                    .push_options()
+                    .map(|s| s.to_owned())
+                    .collect::<Vec<String>>(),
+            ),
+            dev_version_ext: Some(empty.dev_version_ext().to_owned()),
+            dev_version: Some(empty.dev_version()),
+            shared_version: Some(empty.shared_version()),
+            consolidate_commits: Some(empty.consolidate_commits()),
+            consolidate_pushes: Some(empty.consolidate_pushes()),
+            pre_release_commit_message: Some(empty.pre_release_commit_message().to_owned()),
+            post_release_commit_message: Some(empty.post_release_commit_message().to_owned()),
+            pre_release_replacements: Some(empty.pre_release_replacements().to_vec()),
+            post_release_replacements: Some(empty.post_release_replacements().to_vec()),
+            pre_release_hook: empty.pre_release_hook().cloned(),
+            tag_message: Some(empty.tag_message().to_owned()),
+            tag_prefix: None, // Skipping, its location dependent
+            tag_name: Some(empty.tag_name().to_owned()),
+            tag: Some(empty.tag()),
+            enable_features: Some(empty.enable_features().to_vec()),
+            enable_all_features: Some(empty.enable_all_features()),
+            dependent_version: Some(empty.dependent_version()),
+        }
+    }
+
     pub fn update(&mut self, source: &Config) {
         if let Some(allow_branch) = source.allow_branch.as_deref() {
             self.allow_branch = Some(allow_branch.to_owned());
@@ -62,21 +102,17 @@ impl Config {
         if let Some(registry) = source.registry.as_deref() {
             self.registry = Some(registry.to_owned());
         }
-        if let Some(release) = resolve_bool_arg(source.release, source.disable_release) {
+        if let Some(release) = source.release {
             self.release = Some(release);
-            self.disable_release = None;
         }
-        if let Some(publish) = resolve_bool_arg(source.publish, source.disable_publish) {
+        if let Some(publish) = source.publish {
             self.publish = Some(publish);
-            self.disable_publish = None;
         }
-        if let Some(verify) = resolve_bool_arg(source.verify, source.no_verify) {
+        if let Some(verify) = source.verify {
             self.verify = Some(verify);
-            self.no_verify = None;
         }
-        if let Some(push) = resolve_bool_arg(source.push, source.disable_push) {
+        if let Some(push) = source.push {
             self.push = Some(push);
-            self.disable_push = None;
         }
         if let Some(push_options) = source.push_options.as_deref() {
             self.push_options = Some(push_options.to_owned());
@@ -84,9 +120,8 @@ impl Config {
         if let Some(dev_version_ext) = source.dev_version_ext.as_deref() {
             self.dev_version_ext = Some(dev_version_ext.to_owned());
         }
-        if let Some(dev_version) = resolve_bool_arg(source.dev_version, source.no_dev_version) {
+        if let Some(dev_version) = source.dev_version {
             self.dev_version = Some(dev_version);
-            self.no_dev_version = None;
         }
         if let Some(shared_version) = source.shared_version {
             self.shared_version = Some(shared_version);
@@ -121,9 +156,8 @@ impl Config {
         if let Some(tag_name) = source.tag_name.as_deref() {
             self.tag_name = Some(tag_name.to_owned());
         }
-        if let Some(tag) = resolve_bool_arg(source.tag, source.disable_tag) {
+        if let Some(tag) = source.tag {
             self.tag = Some(tag);
-            self.disable_tag = None;
         }
         if let Some(enable_features) = source.enable_features.as_deref() {
             self.enable_features = Some(enable_features.to_owned());
@@ -160,26 +194,26 @@ impl Config {
     }
 
     pub fn release(&self) -> bool {
-        resolve_bool_arg(self.release, self.disable_release).unwrap_or(true)
+        self.release.unwrap_or(true)
     }
 
     pub fn publish(&self) -> bool {
-        resolve_bool_arg(self.publish, self.disable_publish).unwrap_or(true)
+        self.publish.unwrap_or(true)
     }
 
     pub fn verify(&self) -> bool {
-        resolve_bool_arg(self.verify, self.no_verify).unwrap_or(true)
+        self.verify.unwrap_or(true)
     }
 
     pub fn push(&self) -> bool {
-        resolve_bool_arg(self.push, self.disable_push).unwrap_or(true)
+        self.push.unwrap_or(true)
     }
 
-    pub fn push_options(&self) -> &[String] {
+    pub fn push_options(&self) -> impl Iterator<Item = &str> {
         self.push_options
             .as_ref()
-            .map(|v| v.as_ref())
-            .unwrap_or(&[])
+            .into_iter()
+            .flat_map(|v| v.iter().map(|s| s.as_str()))
     }
 
     pub fn dev_version_ext(&self) -> &str {
@@ -187,7 +221,7 @@ impl Config {
     }
 
     pub fn dev_version(&self) -> bool {
-        resolve_bool_arg(self.dev_version, self.no_dev_version).unwrap_or(false)
+        self.dev_version.unwrap_or(false)
     }
 
     pub fn shared_version(&self) -> bool {
@@ -250,7 +284,7 @@ impl Config {
     }
 
     pub fn tag(&self) -> bool {
-        resolve_bool_arg(self.tag, self.disable_tag).unwrap_or(true)
+        self.tag.unwrap_or(true)
     }
 
     pub fn enable_features(&self) -> &[String] {
@@ -311,16 +345,15 @@ impl Command {
     }
 }
 
-arg_enum! {
-    #[derive(Debug, Clone, Copy, PartialEq, Eq, Serialize, Deserialize)]
-    #[serde(rename_all = "kebab-case")]
-    pub enum DependentVersion {
-        Upgrade,
-        Fix,
-        Error,
-        Warn,
-        Ignore,
-    }
+#[derive(Debug, Clone, Copy, PartialEq, Eq, Serialize, Deserialize, clap::ArgEnum)]
+#[serde(rename_all = "kebab-case")]
+#[clap(rename_all = "kebab-case")]
+pub enum DependentVersion {
+    Upgrade,
+    Fix,
+    Error,
+    Warn,
+    Ignore,
 }
 
 impl Default for DependentVersion {
@@ -366,6 +399,64 @@ struct CargoMetadata {
     release: Option<Config>,
 }
 
+pub fn load_workspace_config(
+    args: &crate::args::ReleaseOpt,
+    ws_meta: &cargo_metadata::Metadata,
+) -> Result<Config, FatalError> {
+    let mut release_config = Config::default();
+
+    if !args.isolated {
+        let cfg = resolve_workspace_config(ws_meta.workspace_root.as_std_path())?;
+        release_config.update(&cfg);
+    }
+
+    if let Some(custom_config_path) = args.custom_config.as_ref() {
+        // when calling with -c option
+        let cfg = resolve_custom_config(Path::new(custom_config_path))?.unwrap_or_default();
+        release_config.update(&cfg);
+    }
+
+    release_config.update(&args.config.to_config());
+    Ok(release_config)
+}
+
+pub fn load_package_config(
+    args: &crate::args::ReleaseOpt,
+    ws_meta: &cargo_metadata::Metadata,
+    pkg: &cargo_metadata::Package,
+) -> Result<Config, FatalError> {
+    let manifest_path = pkg.manifest_path.as_std_path();
+
+    let mut release_config = Config::default();
+
+    if !args.isolated {
+        let cfg = resolve_config(ws_meta.workspace_root.as_std_path(), manifest_path)?;
+        release_config.update(&cfg);
+    }
+
+    if let Some(custom_config_path) = args.custom_config.as_ref() {
+        // when calling with -c option
+        let cfg = resolve_custom_config(Path::new(custom_config_path))?.unwrap_or_default();
+        release_config.update(&cfg);
+    }
+
+    release_config.update(&args.config.to_config());
+
+    // the publish flag in cargo file
+    let cargo_file = crate::cargo::parse_cargo_config(manifest_path)?;
+    if !cargo_file
+        .get("package")
+        .and_then(|f| f.as_table())
+        .and_then(|f| f.get("publish"))
+        .and_then(|f| f.as_bool())
+        .unwrap_or(true)
+    {
+        release_config.publish = Some(false);
+    }
+
+    Ok(release_config)
+}
+
 pub fn dump_config(
     args: &crate::args::ReleaseOpt,
     output_path: &std::path::Path,
@@ -386,52 +477,13 @@ pub fn dump_config(
                 .iter()
                 .find(|p| p.id == *root_id)
                 .expect("root should always be present");
-            let manifest_path = pkg.manifest_path.as_std_path();
-
-            let mut release_config = Config::default();
-
-            if !args.isolated {
-                let cfg = resolve_config(ws_meta.workspace_root.as_std_path(), manifest_path)?;
-                release_config.update(&cfg);
-            }
-
-            if let Some(custom_config_path) = args.custom_config.as_ref() {
-                // when calling with -c option
-                let cfg = resolve_custom_config(Path::new(custom_config_path))?.unwrap_or_default();
-                release_config.update(&cfg);
-            }
-
-            release_config.update(&args.config.to_config());
-
-            // the publish flag in cargo file
-            let cargo_file = crate::cargo::parse_cargo_config(manifest_path)?;
-            if !cargo_file
-                .get("package")
-                .and_then(|f| f.as_table())
-                .and_then(|f| f.get("publish"))
-                .and_then(|f| f.as_bool())
-                .unwrap_or(true)
-            {
-                release_config.publish = Some(false);
-                release_config.disable_publish = None;
-            }
 
+            let mut release_config = Config::from_defaults();
+            release_config.update(&load_package_config(args, &ws_meta, pkg)?);
             release_config
         } else {
-            let mut release_config = Config::default();
-
-            if !args.isolated {
-                let cfg = resolve_workspace_config(ws_meta.workspace_root.as_std_path())?;
-                release_config.update(&cfg);
-            }
-
-            if let Some(custom_config_path) = args.custom_config.as_ref() {
-                // when calling with -c option
-                let cfg = resolve_custom_config(Path::new(custom_config_path))?.unwrap_or_default();
-                release_config.update(&cfg);
-            }
-
-            release_config.update(&args.config.to_config());
+            let mut release_config = Config::from_defaults();
+            release_config.update(&load_workspace_config(args, &ws_meta)?);
             release_config
         };
 
@@ -588,18 +640,6 @@ pub fn resolve_config(workspace_root: &Path, manifest_path: &Path) -> Result<Con
     Ok(config)
 }
 
-fn resolve_bool_arg(yes: Option<bool>, no: Option<bool>) -> Option<bool> {
-    match (yes, no) {
-        (Some(_), Some(_)) => panic!("`disable_` flags are deprecated and can't be set at the same time with their positive versions"),
-        (Some(yes), None) => Some(yes),
-        (None, Some(no)) => {
-            log::warn!("Negative config values are deprecated (`no_`, `disable_`)");
-            Some(!no)
-        },
-        (None, None) => None,
-    }
-}
-
 #[cfg(test)]
 mod test {
     use super::*;
